import '../chunk-5USKE2QT.js';
import { useMemo, useRef } from 'react';
import { isFunction } from '../utils.js';
import usePrevious from './use-previous.js';
import useUpdate from './use-update.js';

function useControlledState(option) {
  const { defaultValue, value, onChange, beforeValue, postValue } = option;
  const isControlled = Object.prototype.hasOwnProperty.call(option, "value") && typeof value !== "undefined";
  const initialValue = useMemo(() => {
    if (isControlled) {
      return value;
    }
    if (defaultValue !== void 0) {
      return isFunction(defaultValue) ? defaultValue() : defaultValue;
    }
  }, []);
  const stateRef = useRef(initialValue);
  if (isControlled) {
    stateRef.current = value;
  }
  const previousState = usePrevious(stateRef.current);
  if (postValue) {
    const post = postValue(stateRef.current, previousState);
    if (post) {
      stateRef.current = post;
    }
  }
  const update = useUpdate();
  const triggerChange = (newValue) => {
    let r = isFunction(newValue) ? newValue(stateRef.current) : newValue;
    if (beforeValue) {
      const before = beforeValue(r, stateRef.current);
      if (before) {
        r = before;
      }
    }
    if (onChange) {
      onChange(r, stateRef.current);
    }
    if (!isControlled) {
      stateRef.current = r;
      update();
    }
  };
  return [stateRef.current, triggerChange, previousState];
}

export { useControlledState as default };
